import time
import random
from pathlib import Path
from typing import Optional

import uiautomation as uia
import pyautogui

from .utils import *

# from wx.anti_detection import anti_detection_start


class WeChat:
    VERSION: str = "4.x"

    def __init__(self, init_msglist: bool = True, chat_with: str | None = None) -> None:
        """微信UI自动化实例"""
        self.WXwindow: uia.WindowControl = uia.WindowControl(searchDepth=3, Name="微信")
        if not self.WXwindow.Exists(0.1):
            uia.SendKeys("{Ctrl}{Alt}w")
        # 日志
        self.logger = get_logger()
        # 清理缓存
        CacheFolder: Path = Path("wxdata") / "cache"
        for file in CacheFolder.iterdir():
            if file.is_file():
                file.unlink()

        self._show()
        self.RuntimeID2Data: dict = {}
        self.Runtimes_Msg: list = []
        self.AllMsgList: list = []
        self.A_Search: uia.EditControl = self.WXwindow.EditControl(Name="搜索")
        self.B_MsgList: uia.ListControl = self.WXwindow.ListControl(Name="消息")
        self.A_contacts: uia.ListControl = self.WXwindow.ListControl(Name="会话")
        self.possible_fastest_message_sending_speed = 0.5
        self.MsgEditorBox = self.WXwindow.EditControl(AutomationId="chat_input_field")
        self.SaveTo = 0
        self.TheLastRuntimeID: Optional[str] = None
        if chat_with is not None:
            print(self.ChatWith(chat_with))
        if init_msglist:
            self.InitGetAllMessage()

    def send_enter(self) -> None:
        """按下回车键"""
        editbox = self.MsgEditorBox
        editbox.SendKeys("{ENTER}")

    def ChatWith(self, contact_name: str) -> tuple[str, int] | tuple[None, None]:
        """切换聊天列表
        Args:
            contact_name:联系人的名字
        """
        for i, contact in enumerate(self.A_contacts.GetChildren()):
            print(str(contact.Name).split(" "), contact_name)
            if str(contact.Name).split(" ")[0] == contact_name:
                contact.Click()
                return (contact.Name, i)
        return (None, None)

    def SendFiles(self, filepath: str | list[str], who: str | None = None) -> bool:
        """向当前聊天窗口发送文件

        Args:
            filepath (str|list): 要复制文件的绝对路径
            who (str): 要发送给谁，如果为None，则发送到当前聊天页面。  *最好完整匹配，优先使用备注

        Returns:
            bool: 是否成功发送文件
        """
        raise NotImplementedError("SendFiles方法未实现")

    # def GetAllMessage(
    #     self,
    #     savepic: bool = False,
    #     savefile: bool = False,
    #     savevoice: bool = False,
    #     readto: int = 0,
    # ) -> list:

    def InitGetAllMessage(self) -> None:
        """
        # To Fix
        获取所有消息的时候,RuntimeID可能重复，导致错误
        ## 方法
        1. 获取所有消息的RuntimeID（OK）
        2. 在每4个控件就生成MSG对象防止RuntimeID重复
        3. 获取每个控件的发送者（Bug, can't run）
        """
        self.move_to_msglist()
        self.LoadMoreMessage()
        self.logger.info("开始获取所有消息")
        Msg = self.B_MsgList
        # 按顺序的RuntimeId列表
        NoMore: int = 0
        while True:
            first_child = Msg.GetFirstChildControl()
            BeforeFirst = first_child.GetRuntimeId() if first_child else None
            current_first = Msg.GetFirstChildControl()
            current_first_id = current_first.GetRuntimeId() if current_first else None
            if BeforeFirst == current_first_id:
                NoMore += 1
                if NoMore > 10:  # 确保没有更多消息
                    break
            else:
                NoMore = 0
            IntheViewContralAllRuntimeID: list = []
            msg_children = Msg.GetChildren()
            if NoMore < 8:
                msg_children = msg_children[:-1]
            for i in msg_children:
                sender: str = ""
                time.sleep(0.1)
                sender = GetSender(i)
                IntheViewContralAllRuntimeID.append(i.GetRuntimeId())
                self.RuntimeID2Data[str(i.GetRuntimeId())] = [i.Name, sender]
            self.Runtimes_Msg = merge_lists(
                self.Runtimes_Msg, IntheViewContralAllRuntimeID
            )
            wheel_control(Msg, wheel_range=[-1500, -1000])
            if len(self.Runtimes_Msg) > self.SaveTo + 4:
                self.UpdataMsgList()
        last_child = self.B_MsgList.GetLastChildControl()
        if last_child is not None:
            self.TheLastRuntimeID = last_child.GetRuntimeId()

    def UpdataMsgList(self) -> None:
        # 将Runtimes_Msg中的消息整理为ALlMsgList，并清空Runtimes_Msg
        for index, runtime_id in enumerate(
            self.Runtimes_Msg[self.SaveTo :], start=len(self.AllMsgList) + 1
        ):
            data = self.RuntimeID2Data.get(str(runtime_id))
            if data:
                content, sender = data
                msg = MSG(index=index, sender=sender, content=content)
                self.AllMsgList.append(msg)
        self.SaveTo = len(self.Runtimes_Msg)

    def GetAllMessage(self) -> list[MSG]:
        self.UpdataMsgList()
        return self.AllMsgList

    def LoadMoreMessage(self) -> None:
        NoMore = 0
        while True:
            first_child = self.B_MsgList.GetFirstChildControl()
            if first_child is None or first_child.GetRuntimeId() is None:
                break
            BeforeFirst = first_child.GetRuntimeId()
            wheel_control(self.B_MsgList, wheel_range=[4900, 6000])
            current_first = self.B_MsgList.GetFirstChildControl()
            current_first_id = current_first.GetRuntimeId() if current_first else None
            if BeforeFirst == current_first_id:
                NoMore += 1
                if NoMore > 3:
                    break
            else:
                NoMore = 0

    def GetAllFriends(self, keywords: str | None = None) -> NotImplementedError:
        """获取所有好友列表
        Args:
            keywords (str, optional): 搜索关键词，只返回包含关键词的好友列表

        Returns:
            list: 所有好友列表
        """
        return NotImplementedError("GetAllFriends无法使用")

    def get_new_message(self) -> None:
        # 监听新消息
        last_child = self.B_MsgList.GetLastChildControl()
        LastRuntimeID = last_child.GetRuntimeId() if last_child else None
        if LastRuntimeID is not None and LastRuntimeID != self.TheLastRuntimeID:
            self.Runtimes_Msg.append(LastRuntimeID)
            control: uia.Control = last_child  # type: ignore
            sender: str = GetSender(control)

            self.RuntimeID2Data[str(control.GetRuntimeId())] = [
                control.Name,
                sender,
            ]
            self.TheLastRuntimeID = LastRuntimeID
        else:
            time.sleep(self.possible_fastest_message_sending_speed)
        if random.randint(0, 50) == 20:
            # anti_detection_start("000000")
            pass

    def _checkversion(self):
        pass

    def _show(self) -> None:
        self.WXwindow.SetTopmost(True)

    def _refresh(self) -> None:
        self.WXwindow.SendKeys("{Ctrl}{Alt}w")
        self.WXwindow.SendKeys("{Ctrl}{Alt}w")
        self._show()

    def CheckNewMessage(self):
        """是否有新消息"""
        pass

    def DownloadImage(self, imcontrol: uia.ImageControl):
        """
        Args:
            imcontrol (uia.ImageControl): 图片控件

        Returns:
            imgpath (str): 图片路径
        """
        raise NotImplementedError("DownloadImage无法使用")

    def GetSessionList(self, reset: bool = False, newmessage: bool = False):
        """获取当前聊天列表中的所有聊天对象

        Args:
            reset (bool): 是否重置SessionItemList
            newmessage (bool): 是否只获取有新消息的聊天对象

        Returns:
            SessionList (dict): 聊天对象列表，键为聊天对象名，值为新消息条数
        """
        raise NotImplementedError("GetSessionList无法使用")

    def SendMsg(
        self,
        msg: str,
        clear: bool = True,
        send: bool = True,
    ) -> None:
        """发送文本消息

        Args:
            msg (str): 要发送的文本消息
            who (str): 要发送给谁，如果为None，则发送到当前聊天页面。  *最好完整匹配，优先使用备注
            clear (bool, optional): 是否清除原本的内容，
            send (bool, optional): 是否发送，默认为True
        """
        self._show()
        if not self.MsgEditorBox.HasKeyboardFocus:
            self.MsgEditorBox.Click(simulateMove=False)
        if clear:
            self.MsgEditorBox.SendKeys("{Ctrl}a", waitTime=0)

        if msg:
            t0 = time.time()
            while True:
                if time.time() - t0 > 10:
                    self.logger.error(
                        f"发送消息超时 --> {self.MsgEditorBox.Name} - {msg}"
                    )
                SetClipboardText(msg)
                self.MsgEditorBox.SendKeys("{Ctrl}v")
                if self.MsgEditorBox.GetValuePattern().Value:
                    break
        if send:
            time.sleep(random.random())  # anti detection
            self.MsgEditorBox.SendKeys("{Enter}")

    def GetWeiXinID(self, index=-1):
        MsgControl = self.B_MsgList.GetChildren()[index]
        rect = MsgControl.BoundingRectangle
        left = rect.left
        top = rect.top
        pyautogui.click(left + 40, top + 30)
        automation_id = (
            "right_v_view.user_info_center_view.basic_line_view.ContactProfileTextView"
        )
        time.sleep(0.5)
        weixin = uia.WindowControl(Name="Weixin")
        WeiXinId = (
            weixin.TextControl(
                AutomationId=automation_id, ClassName="mmui::ContactProfileTextView"
            )
            .GetParentControl()
            .GetParentControl()
            .GetChildren()[1]
            .GetChildren()[-1]
            .Name
        )
        return WeiXinId

    def move_to_msglist(self):
        """移动到消息列表"""
        rect = self.B_MsgList.BoundingRectangle
        width = rect.right - rect.left
        height = rect.bottom - rect.top
        center_x = int(rect.left) + width / 2
        center_y = int(rect.top) + height / 2
        pyautogui.moveTo(int(center_x), int(center_y), duration=1 + random.random())



import os.path
import re
import random
import time

import pyperclip
import pyautogui
import uiautomation as uia
from PIL import Image, UnidentifiedImageError
from loguru import logger
from dotenv import load_dotenv


def get_logger():
    logger.remove()
    load_dotenv()
    if os.getenv("Logs") is None:
        raise RuntimeError("Error to get log path")
    else:
        if not os.path.exists(str(os.getenv("Logs"))):
            raise RuntimeError(
                f"Log file doesn't exists, Config file path:{os.getenv('Logs')}"
            )
        else:
            logger.add(str(os.getenv("Logs")))
    return logger


logger = get_logger()


def voice_msg_processor(msg_content) -> None | dict:
    """处理音频消息
    Args:
        msg_content(str):消息的内容
    """
    msg = msg_content
    pattern = r'"语言(\d+)"秒(.*)'
    match = re.search(pattern, msg)
    if match:
        if (match.group(1) is None) or (match.group(2) is None):
            return None
        else:
            return {"time": match.group(1), "msg": int(match.group(2))}
    else:
        return None


def SetClipboardText(text: str):
    pyperclip.copy(text)


class MSG:
    """
    wx的消息
    """

    def __init__(
        self, index: int, sender: str, content: str, auto_process_voice_msg: bool = True
    ) -> None:
        self.sender = sender
        self.content = content
        self.index = index
        if not auto_process_voice_msg:
            return
        voice_msg = voice_msg_processor(self.content)
        if voice_msg is not None:  # 避免重复调用
            self.time: int = int(voice_msg["time"])
            self.content: str = str(voice_msg["content"])

    def __str__(self):
        return f"MSG(index={self.index}, sender={self.sender}, content={self.content})"

    def __repr__(self):
        return f"MSG(index={self.index}, sender={self.sender}, content={self.content})"


def merge_lists(a: list, b: list) -> list:
    """
    合并两个列表,去除重复部分
    Args:
        a(list):第一个列表
        b(list):第二个列表
    Example:
        ```
        merge_lists([1,2,3,4,5],[3,4,5,6,7,8])
        ```
        Return [1,2,3,4,5,6,7,8]
    """

    def find_overlap(a: list, b: list) -> int:
        max_len = min(len(a), len(b))
        # Check for end of a and start of b overlap
        max_overlap = 0
        for overlap in range(max_len, -1, -1):
            if overlap == 0:
                break
            if a[-overlap:] == b[:overlap]:
                max_overlap = overlap
                break
        return max_overlap

    overlap = find_overlap(a, b)
    if overlap > 0:
        merged = a + b[overlap:]
    else:
        overlap_start = find_overlap(b, a)
        if overlap_start > 0:
            merged = b + a[overlap_start:]
        else:
            merged = a + b
    return merged


def capture_control_image(control: uia.Control):
    """
    根据控件对象截取屏幕上的控件图像。
    """
    rect = control.BoundingRectangle
    left = rect.left
    top = rect.top
    right = rect.right
    bottom = rect.bottom
    width = right - left
    height = bottom - top
    screenshot: Image.Image = pyautogui.screenshot(region=(left, top, width, height))
    return screenshot


def is_fully_visible(control: uia.Control):
    """
    判断控件是否完全可见
    :param control: uiautomation控件对象
    :return: True如果控件完全可见，False否则
    """
    # 首先检查控件是否在屏幕上
    if control.IsOffscreen:
        return False

    # 获取控件的边界矩形
    try:
        rect = control.BoundingRectangle
        if not rect:
            return False

        # 使用Rect对象的属性而不是下标访问
        left, top, right, bottom = rect.left, rect.top, rect.right, rect.bottom

        # 检查控件是否有有效的尺寸
        if right <= left or bottom <= top:
            return False

        # 获取桌面窗口（用于比较屏幕尺寸）
        desktop = uia.GetRootControl()
        desktop_rect = desktop.BoundingRectangle

        # 检查控件是否完全在桌面可见区域内
        return (
            left >= desktop_rect.left
            and top >= desktop_rect.top
            and right <= desktop_rect.right
            and bottom <= desktop_rect.bottom
        )
    except Exception as e:
        logger.error(f"检查可见性时出错: {e}")
        return False


def image_contains_color(image_path, tolerance=0) -> bool:
    """
    检查图片中是否包含(149, 236, 105)。

    :param image_path: 图片路径
    :param tolerance: 颜色容差
    :return: 如果图片中包含目标颜色，返回True，否则返回False
    """
    try:
        image = Image.open(image_path)
    except UnidentifiedImageError as e:
        logger.error(e)
        return False
    image = image.convert("RGB")
    width, height = image.size
    count: int = 0
    for x in range(width):
        for y in range(height):
            pixel_color = image.getpixel((x, y))
            diff = sum(
                abs(a - b) for a, b in zip(pixel_color, (149, 236, 105))
            )  # 计算当前颜色与目标颜色之间的差值

            if diff <= tolerance:  # 如果差值小于容差，则认为颜色匹配
                count += 1
            if count > 50:
                return True
    return False


def GetSender(control) -> str:
    save_path = f"wxdata\\cache\\{str(control.GetRuntimeId())}.png"
    if (
        (control.Name != "图片")  # 不是图片
        and (str(control.AutomationId) != "")  # 不是时间
        and (is_fully_visible(control))  # 控件完全可见
        and (control.Name != "文件")  # 不是文件
    ):
        if not os.path.exists(save_path):
            screenshot = capture_control_image(control)
            screenshot.save(save_path)
        if image_contains_color(save_path):
            sender = "Self"
        else:
            sender = "Other"
    elif str(control.AutomationId) == "":
        sender = "SYS"
    else:
        sender = ""
    return sender


def wheel_control(
    control: uia.Control, times: int = 5, wheel_range: list[int] = [300, 400]
) -> None:
    """滚轮控制
    Args:
        control(uiautomation.Control):要滚动的控件
        times(int):滚动的分的次数
        wheel_range(list):随机范围，例如[300, 400]
    """
    if times <= 0:  # 避免除零错误
        logger.warning("times 参数必须大于 0")
        return
    time_list = [random.random() for _ in range(times)]
    control.SetFocus()
    for i in range(times):
        pyautogui.scroll(
            clicks=random.randint(
                int(wheel_range[0] / times), int(wheel_range[1] / times)
            )
        )
        time.sleep(time_list[i] / 20)


if __name__ == "__main__":
    print(merge_lists([1, 2], [1, 2]))



from random import random
import pyautogui
import time
import json
from loguru import logger


def gather(duration, data_id="000000") -> dict[str, list]:
    time.sleep(2)
    print("Start to gather anti detection data")
    start = time.time()  # record start time
    move_history: list = []
    while (time.time() - start) < duration:
        x, y = pyautogui.position()
        move_history.append([x, y])
        # print(x, y)
        time.sleep(1 / 200)  # 200 times per second
    return {data_id: move_history}


def anti_detection_start(data_key: str, about_duration=5) -> None:
    with open("wx\\anti_detection.json", "r", encoding="utf-8") as f:
        anti_detection_data: dict = json.load(f)
        if anti_detection_start is None:
            logger
    for x, y in anti_detection_data[data_key]:
        pyautogui.moveTo((int(x * random()), int(y * random())))


if __name__ == "__main__":
    # with open("wx\\anti_detection.json", "w", encoding="utf-8") as f:
    # json.dump(gather(5), f)
    anti_detection_start("000000")
